#ifndef YASERVER_MYSQLCONNPOOL_H_
#define YASERVER_MYSQLCONNPOOL_H_

#include <mysql.h>
#include <list>

using namespace std;

class CMysqlConn{
public:
	CMysqlConn(const char *option, const char *host, const char *user,
		const char *password, const char* dbname, unsigned int port, 
		const char *unix_socket, unsigned long clientflag, bool &state);
	virtual ~CMysqlConn();
public:
	// ӵݿ
	int SetOption();
	int Open();
	// رݿclei
	int Close();
	// ݿǷ
	bool IsOpen() const;
	// query

	MYSQL_RES * Execute(const char * strSQL, int &err);
	MYSQL_RES * MultiExecute( const char * strSQL, int &err);

private:
	MYSQL * m_pConn;
	bool m_bValid; // ѱʼɹʶ
	bool m_bOpen;

	const char * m_option;
	const char * m_host;
	const char * m_user;
	const char * m_pwd;
	const char * m_db;
	unsigned int m_port;
	const char * m_unix_socket;
	unsigned long m_clientflag;
};

typedef std::list<CMysqlConn*> DBConnectList;

class CMysqlConnPool
{
public:
	// ȡʵָ
	static CMysqlConnPool &Instanse(){
		static CMysqlConnPool m_pInstanse;
		return m_pInstanse;
	};

	// ʼ
	int InitAllConn();

	// ر
	void CloseAllConn();
	// ȡһ
	CMysqlConn* GetNewConn();
	// Ӹж
	int RestoreConn(CMysqlConn* pDBEngine);
	void Init(const char* szOption, const char *szHost, const char *szUser, const char *szPwd, const char *szDbName,
		unsigned int nPort, const char * szUnixSocket, unsigned long nClientFlag);
private:
	// Ψһʵ
	CMysqlConnPool();
	virtual ~CMysqlConnPool();
	// һ
	int InitNewConn();
	// رһ
	void CloseConn(CMysqlConn* pDBEngine);
	// ֹͣ߳
	void StopThread();

	// жǷҪֹͣ
	BOOL IsNeedStop();
	BOOL IsNeedConnection();
	// ΪӳԪ
	friend class CDBConnGuard;

	// ݿӶ
	DBConnectList m_listIdleConnection;
	// ʹõݿ
	DBConnectList m_listBusyConnection;

	// бٽ
	CRITICAL_SECTION m_csIdleConnList;
	CRITICAL_SECTION m_csBusyConnList;

	// ָ꣺С
	unsigned int m_nMaxCount;
	unsigned int m_nMinCount;

	// ݿϢ
	std::string m_strHost;
	std::string m_strUser;
	std::string m_strPwd;
	std::string m_strUnixSocket;
	std::string m_strOption;
	std::string m_strDbName;
	//const char * m_strHost;
	//const char * m_strUser;
	//const char * m_strPwd;
	//const char * m_strUnixSocket;
	//const char * m_strOption;
	//const char * m_strDbName;
	unsigned int m_nPort;
	unsigned long m_nClientFlag;

	// ά߳
	HANDLE m_hMaintanceThread; // ߳̾
	HANDLE m_hHaveData; // ź

	BOOL m_bNeedStop; // ߳ͣı־λ
	BOOL m_bNeedConnection; // Ҫӵı־
	static DWORD WINAPI thread_run( LPVOID pdata);
};

class CDBConnGuard{
public:
	CDBConnGuard(CMysqlConn* &pDbConn){
		pDbConn = CMysqlConnPool::Instanse().GetNewConn();
		m_pDbConn = pDbConn;
	}
	virtual ~CDBConnGuard()
	{
		CMysqlConnPool::Instanse().RestoreConn(m_pDbConn);
	}
private:
	CMysqlConn * m_pDbConn;
};

#define GetConnPool() CMysqlConnPool::Instanse()

#endif //YASERVER_MYSQLCONNPOOL_H_