#include "stdafx.h"
#include"SyncManager.h"
#include <deque>
#include <Eigen/Dense>

HostServer::SyncManager::SyncManager()
{
    init();
}

void HostServer::SyncManager::init()
{
	InitializeCriticalSection(&m_csSyncTime);
	InitializeCriticalSection(&m_csCalcLiner);
    _anchors.swap(unordered_map<unsigned long long, Position>());
	//_syncTimeMsgs.clear();
	//_historySync.clear();
    _distance.swap(unordered_map<unsigned long long,unordered_map<unsigned long long, double>>());
}

void HostServer::SyncManager::analyzeSyncMsg(SyncTimeMsg &msg)
{
	EnterCriticalSection(&m_csSyncTime);
	int idx = FindSyncTimeMsg(msg.RootIdCode(), msg.SyncNum());

	if( -1 == idx){ // ûҵ
		// ʱͬϢİ汾ƣɾӵϢ
		if(_syncTimeMsgs[msg.RootIdCode()].size() >= MAX_SYNCTIME_NUM){ // ɾһ
			// й¶
			_syncTimeMsgs[msg.RootIdCode()].pop_front();
		}
		SyncTimeMsgItem it;
		it.SyncNum = msg.SyncNum();
		it.SyncTimeMsgs[msg.LocalIdCode()] = msg;
		_syncTimeMsgs[msg.RootIdCode()].push_back(it);

		idx = _syncTimeMsgs[msg.RootIdCode()].size() - 1;
	}else{
		_syncTimeMsgs[msg.RootIdCode()][idx].SyncTimeMsgs[msg.LocalIdCode()] = msg;
	}

	//
	while(_historySync[msg.RootIdCode()].size() > MAX_SYNCTIME_NUM){
		_historySync[msg.RootIdCode()].pop_front();
	}

    // ʱͬ
	for(auto it :_syncTimeMsgs[msg.RootIdCode()][idx].SyncTimeMsgs)
	{
		updateSync(msg.RootIdCode(), idx, msg.SyncNum(), it.first);
	}
	LeaveCriticalSection(&m_csSyncTime);
}

bool HostServer::SyncManager::updateSync(unsigned long long rootIdCode, int idx, unsigned short SyncNum, unsigned long long localIdCode)
{
	if(-1 == idx){
		return false;
	}
    // ǰ汾ʱͬϢδյ򷵻false
	unordered_map<unsigned long long, SyncTimeMsg>::iterator itSyncTime = _syncTimeMsgs[rootIdCode][idx].SyncTimeMsgs.find(localIdCode);
	if(itSyncTime == _syncTimeMsgs[rootIdCode][idx].SyncTimeMsgs.end()){
		return false;
	}

    SyncTimeMsg &msg = _syncTimeMsgs[rootIdCode][idx].SyncTimeMsgs[localIdCode];

    // ǰΪroot򷵻true
    if(msg.SyncLevel() == 0){
        return true;
    } 

	int idx_synctime = FindSyncTime(rootIdCode, SyncNum);

	// ʱͬѾ򷵻true
	if(-1 != idx_synctime){
		unordered_map<unsigned long long, SyncTime>::iterator itHistSync = _historySync[rootIdCode][idx_synctime].HistSync.find(localIdCode);
		if(itHistSync != _historySync[rootIdCode][idx_synctime].HistSync.end()){
			if(itHistSync->second.TimeDelay())
			return true;
		}
	}

    // ѾյһͬϢ
    if(_syncTimeMsgs[rootIdCode][idx].SyncTimeMsgs.count(msg.UpperIdCode())){
        if(!updateSync(rootIdCode, idx, SyncNum, msg.UpperIdCode())){
            return false;
        }
        SyncTimeMsg &upperMsg = _syncTimeMsgs[rootIdCode][idx].SyncTimeMsgs[msg.UpperIdCode()];
        SyncTime* s = NULL;
        for(auto it(_historySync[rootIdCode].rbegin()); it != _historySync[rootIdCode].rend(); ++it)
        {
            if(it->SyncNum != msg.SyncNum() && it->HistSync.count(localIdCode))
            {
                s = &(it->HistSync.find(localIdCode)->second);
                break;
            }
        }

		idx_synctime = FindSyncTime(rootIdCode, SyncNum);

		if(-1 == idx_synctime){
			SyncTimeItem it;
			it.SyncNum = msg.SyncNum();
			it.HistSync[localIdCode] = SyncTime(msg, upperMsg, s);
			_historySync[rootIdCode].push_back(it);

			idx_synctime = _historySync[rootIdCode].size() - 1;
		}else{
			_historySync[rootIdCode][idx_synctime].HistSync[localIdCode] = SyncTime(msg, upperMsg, s);
		}

        // ʱͬ
        long long upperTimeDelay = 0;
        if(_syncTimeMsgs[rootIdCode][idx].SyncTimeMsgs[msg.UpperIdCode()].SyncLevel() != 0)
        {
            upperTimeDelay = _historySync[rootIdCode][idx_synctime].HistSync[msg.UpperIdCode()].TimeDelay();
        }
        long long sendTime = _historySync[rootIdCode][idx_synctime].HistSync[localIdCode].SendTime();
        long long receiveTime = _historySync[rootIdCode][idx_synctime].HistSync[localIdCode].ReceiveTime();
        long long timeDelay = receiveTime - sendTime - _distance[localIdCode][msg.UpperIdCode()];

        timeDelay += upperTimeDelay;
		if(timeDelay > TIME_MAX){
			timeDelay -= TIME_MAX;
		}
		if(timeDelay + TIME_MAX < 0 ){
			timeDelay += TIME_MAX;
		}
        _historySync[rootIdCode][idx_synctime].HistSync[localIdCode].TimeDelay(timeDelay);
        return true;
    }
    return false;
}

unsigned long long HostServer::SyncManager::calTimeByLinar(TagMsg &tag)
{
	EnterCriticalSection(&m_csCalcLiner);
    // ȡʷ¼tag
    deque<SyncTime> hisSync;
    //ofstream fout("test/linar_513.txt",ios::app);

	unsigned long long rootIdCode = 0;// getRootIdCode(tag.StationIdCode);

    int i = 0;
	int idx = FindSyncTimeMsg(rootIdCode, tag.SyncNum);
	if(-1 != idx){
		unordered_map<unsigned long long, SyncTimeMsg>::iterator it = _syncTimeMsgs[rootIdCode][idx].SyncTimeMsgs.find(tag.StationIdCode);
		if(it != _syncTimeMsgs[rootIdCode][idx].SyncTimeMsgs.end()){
			if(it->second.SyncLevel() == 0){
				LeaveCriticalSection(&m_csCalcLiner);
				return tag.ReceiveTime;
			}
		}
	}
	
	int idx_sync = -1;
    while(hisSync.size() < 2 && i < MAX_CALCLINER_NUM)
    {
        auto syncNum = tag.SyncNum - i;
		idx_sync = FindSyncTime(rootIdCode, syncNum);
		if(-1 != idx_sync){
			if(_historySync[rootIdCode][idx_sync].HistSync.count(tag.StationIdCode)){
				hisSync.push_front(_historySync[rootIdCode][idx_sync].HistSync[tag.StationIdCode]);
			}
		}
        i++;
    }
    // ʷ¼򷵻
    if(hisSync.size() < 2){
		LeaveCriticalSection(&m_csCalcLiner);
		return LLONG_MAX;
	}

	/*unsigned long long root_sync_time = 0;

	for(deque<SyncTime>::iterator it = hisSync.begin();it!=hisSync.end();++it){
		if(it->SyncLevel == 0){
			root_sync_time = it->SendTime;
		}
	}*/

    // Ԥֵ
    long long y1(hisSync.at(0).ReceiveTime() - hisSync.at(0).TimeDelay()),y2(hisSync.at(1).ReceiveTime() - hisSync.at(1).TimeDelay());
    long long x1(hisSync.at(0).RealReceiveTime()), x2(hisSync.at(1).RealReceiveTime()), x3(tag.ReceiveTime);
    unsigned long long res;
    if(x1 > x2)
    {
        x2 += TIME_MAX;
    }
    if(x2 > x3)
    {
        x3 += TIME_MAX;
    }

	if(y1 < 0){ // yֵС0
		y1 += TIME_MAX;
	}

	if(y2 < 0 ){
		y2 += TIME_MAX;
	}

	if(y1 > y2){
		y2 += TIME_MAX;
	}else if(y2-y1 > TIME_MAX){
		y2 -=TIME_MAX;
	}

    Eigen::Matrix3d a;
    a << x1 ,1 , 0,
         x2 ,1 , 0,
         x3, 1, -1;

    Eigen::Vector3d b(y1, y2, 0);
    Eigen::Vector3d X = a.colPivHouseholderQr().solve(b);
    
    res = X(2);
    res &= TIME_MAX;

	LeaveCriticalSection(&m_csCalcLiner);
    return res;
}

void HostServer::SyncManager::updateDistance(unsigned int localId, unsigned char localAntNum, unsigned int upperId, unsigned char uppderAntNum, double d)
{
    unsigned long long lId = SyncHelper::parseId(localId, localAntNum);
    unsigned long long uId = SyncHelper::parseId(upperId, uppderAntNum);
    if(_anchors.count(lId) == 0)
    {
        _anchors[lId] = Position();
    }
    if(_anchors.count(uId) == 0) 
    {
        _anchors[uId] = Position();
    }
    _distance[lId][uId] = d;
    _distance[uId][lId] = d;

    // ɾа汾Ϣ¼
	_historySync.clear();
	_syncTimeMsgs.clear();
    //_historySync.swap(unordered_map<unsigned short, unordered_map<unsigned long long, SyncTime>>());
    //_syncTimeMsgs.swap(unordered_map<unsigned short, unordered_map<unsigned long long, SyncTimeMsg>>());
}

void HostServer::SyncManager::updateAnchor(unsigned int localId, unsigned char localAntNum, double x, double y, double z)
{
    deleteAnchor(localId, localAntNum);

    unsigned long long lId = SyncHelper::parseId(localId, localAntNum);
    _anchors[lId] = Position( x, y, z);
}

void HostServer::SyncManager::deleteAnchor(unsigned int localId, unsigned char localAntNum)
{
    unsigned long long lId = SyncHelper::parseId(localId, localAntNum);
    auto it = _anchors.find(lId);
    if(it == _anchors.end()) return;

    // ɾanchor
    _anchors.erase(it);

    // ɾanchorصľϢ
    _distance.erase(_distance.find(lId));
    for(auto it(_distance.begin()); it != _distance.end(); it++)
    {
        it->second.erase(it->second.find(lId));
        if(it->second.size() == 0)
        {
            it = _distance.erase(it);
        }
    }

    // ɾа汾Ϣ¼
	_historySync.clear();
	_syncTimeMsgs.clear();
    //_historySync.swap(unordered_map<unsigned short, unordered_map<unsigned long long, SyncTime>>());
    //_syncTimeMsgs.swap(unordered_map<unsigned short, unordered_map<unsigned long long, SyncTimeMsg>>());
}

HostServer::SyncManager::~SyncManager()
{
	DeleteCriticalSection(&m_csCalcLiner);
	DeleteCriticalSection(&m_csSyncTime);
}

int HostServer::SyncManager::FindSyncTimeMsg(unsigned long long rootIdCode, unsigned short SyncNum )
{
	int idx = -1;
	for(int i = _syncTimeMsgs[rootIdCode].size() - 1; i >= 0; i--){
		if(_syncTimeMsgs[rootIdCode][i].SyncNum == SyncNum ){
			return i;
		}
	}
	return idx;
}

int HostServer::SyncManager::FindSyncTime(unsigned long long rootIdCode, unsigned short SyncNum )
{
	int idx = -1;
	for(int i = _historySync[rootIdCode].size() - 1; i >= 0; i--){
		if(_historySync[rootIdCode][i].SyncNum == SyncNum ){
			return i;
		}
	}
	return idx;
}
